@@ -157,9 +157,14 @@ second character.
 
 This module exports the following functions:
     match      Match a regular expression pattern at the beginning of a string.
+    fullmatch  Match a regular expression pattern against all of a string.
     search     Search a string for the presence of a pattern.
-    sub        Substitute occurrences of a pattern found in a string.
+    sub        Substitute occurrences of a pattern found in a string using a
+               template string.
+    subf       Substitute occurrences of a pattern found in a string using a
+               format string.
     subn       Same as sub, but also return the number of substitutions made.
+    subfn      Same as subf, but also return the number of substitutions made.
     split      Split a string by the occurrences of a pattern. VERSION1: will
                split at zero-width match; VERSION0: won't split at zero-width
                match.
@@ -212,14 +217,15 @@ This module also defines an exception 'error'.
 """
 
 # Public symbols.
-__all__ = ["compile", "escape", "findall", "finditer", "match", "purge",
-  "search", "split", "splititer", "sub", "subn", "template", "A", "ASCII", "B",
-  "BESTMATCH", "D", "DEBUG", "E", "ENHANCEMATCH", "S", "DOTALL", "F",
-  "FULLCASE", "I", "IGNORECASE", "L", "LOCALE", "M", "MULTILINE", "R",
-  "REVERSE", "T", "TEMPLATE", "U", "UNICODE", "V0", "VERSION0", "V1",
-  "VERSION1", "X", "VERBOSE", "W", "WORD", "error"]
+__all__ = ["compile", "escape", "findall", "finditer", "fullmatch", "match",
+  "purge", "search", "split", "splititer", "sub", "subf", "subfn", "subn",
+  "template", "Scanner", "A", "ASCII", "B", "BESTMATCH", "D", "DEBUG", "E",
+  "ENHANCEMATCH", "S", "DOTALL", "F", "FULLCASE", "I", "IGNORECASE", "L",
+  "LOCALE", "M", "MULTILINE", "R", "REVERSE", "T", "TEMPLATE", "U", "UNICODE",
+  "V0", "VERSION0", "V1", "VERSION1", "X", "VERBOSE", "W", "WORD", "error",
+  "Regex"]
 
-__version__ = "2.4.1"
+__version__ = "2.4.41"
 
 # --------------------------------------------------------------------
 # Public interface.
@@ -231,6 +237,13 @@ def match(pattern, string, flags=0, pos=None, endpos=None, concurrent=None,
     return _compile(pattern, flags, kwargs).match(string, pos, endpos,
       concurrent)
 
+def fullmatch(pattern, string, flags=0, pos=None, endpos=None, concurrent=None,
+  **kwargs):
+    """Try to apply the pattern against all of the string, returning a match
+    object, or None if no match was found."""
+    return _compile(pattern, flags, kwargs).fullmatch(string, pos, endpos,
+      concurrent)
+
 def search(pattern, string, flags=0, pos=None, endpos=None, concurrent=None,
   **kwargs):
     """Search through string looking for a match to the pattern, returning a
@@ -248,6 +261,16 @@ def sub(pattern, repl, string, count=0, flags=0, pos=None, endpos=None,
     return _compile(pattern, flags, kwargs).sub(repl, string, count, pos,
       endpos, concurrent)
 
+def subf(pattern, format, string, count=0, flags=0, pos=None, endpos=None,
+  concurrent=None, **kwargs):
+    """Return the string obtained by replacing the leftmost (or rightmost with a
+    reverse pattern) non-overlapping occurrences of the pattern in string by the
+    replacement format. format can be either a string or a callable; if a string,
+    it's treated as a format string; if a callable, it's passed the match object
+    and must return a replacement string to be used."""
+    return _compile(pattern, flags, kwargs).subf(format, string, count, pos,
+      endpos, concurrent)
+
 def subn(pattern, repl, string, count=0, flags=0, pos=None, endpos=None,
   concurrent=None, **kwargs):
     """Return a 2-tuple containing (new_string, number). new_string is the string
@@ -260,6 +283,18 @@ def subn(pattern, repl, string, count=0, flags=0, pos=None, endpos=None,
     return _compile(pattern, flags, kwargs).subn(repl, string, count, pos,
       endpos, concurrent)
 
+def subfn(pattern, format, string, count=0, flags=0, pos=None, endpos=None,
+  concurrent=None, **kwargs):
+    """Return a 2-tuple containing (new_string, number). new_string is the string
+    obtained by replacing the leftmost (or rightmost with a reverse pattern)
+    non-overlapping occurrences of the pattern in the source string by the
+    replacement format. number is the number of substitutions that were made. format
+    can be either a string or a callable; if a string, it's treated as a format
+    string; if a callable, it's passed the match object and must return a
+    replacement string to be used."""
+    return _compile(pattern, flags, kwargs).subfn(format, string, count, pos,
+      endpos, concurrent)
+
 def split(pattern, string, maxsplit=0, flags=0, concurrent=None, **kwargs):
     """Split the source string by the occurrences of the pattern, returning a
     list containing the resulting substrings.  If capturing parentheses are used
@@ -309,7 +344,7 @@ def escape(pattern, special_only=False):
         s = []
         if special_only:
             for c in pattern:
-                if c in NONLITERAL:
+                if c in _METACHARS:
                     s.append("\\")
                     s.append(c)
                 elif c == "\x00":
@@ -318,7 +353,7 @@ def escape(pattern, special_only=False):
                     s.append(c)
         else:
             for c in pattern:
-                if c in ALNUM:
+                if c in _ALNUM:
                     s.append(c)
                 elif c == "\x00":
                     s.append("\\000")
@@ -331,7 +366,7 @@ def escape(pattern, special_only=False):
         s = []
         if special_only:
             for c in pattern:
-                if chr(c) in NONLITERAL:
+                if chr(c) in _METACHARS:
                     s.extend(b"\\")
                     s.append(c)
                 elif c == 0:
@@ -340,7 +375,7 @@ def escape(pattern, special_only=False):
                     s.append(c)
         else:
             for c in pattern:
-                if chr(c) in ALNUM:
+                if chr(c) in _ALNUM:
                     s.append(c)
                 elif c == 0:
                     s.extend(b"\\000")
@@ -355,109 +390,121 @@ def escape(pattern, special_only=False):
 
 import _regex_core
 import _regex
+from threading import RLock as _RLock
 from _regex_core import *
+from _regex_core import (_ALL_VERSIONS, _ALL_ENCODINGS, _FirstSetError,
+  _UnscopedFlagSet, _check_group_features, _compile_firstset,
+  _compile_replacement, _flatten_code, _fold_case, _get_required_string,
+  _parse_pattern, _shrink_cache)
+from _regex_core import (ALNUM as _ALNUM, Info as _Info, OP as _OP, Source as
+  _Source, Fuzzy as _Fuzzy)
 
 # Version 0 is the old behaviour, compatible with the original 're' module.
 # Version 1 is the new behaviour, which differs slightly.
 
 DEFAULT_VERSION = VERSION0
 
-ALL_VERSIONS = VERSION0 | VERSION1
-ALL_ENCODINGS = ASCII | LOCALE | UNICODE
+_METACHARS = frozenset("()[]{}?*+|^$\\.")
 
 _regex_core.DEFAULT_VERSION = DEFAULT_VERSION
 
 # Caches for the patterns and replacements.
 _cache = {}
+_cache_lock = _RLock()
 _named_args = {}
+_replacement_cache = {}
 
 # Maximum size of the cache.
 _MAXCACHE = 500
+_MAXREPCACHE = 500
 
-def _compile(pattern, flags=0, kwargs=None):
+def _compile(pattern, flags=0, kwargs={}):
     "Compiles a regular expression to a PatternObject."
-    # We're checking in this order because pattern_type isn't defined when
-    # _compile() is first called, with a string pattern, but only after the
-    # support objects are defined.
-    if isinstance(pattern, (str, bytes)):
+    try:
+        # Do we know what keyword arguments are needed?
+        args_key = pattern, type(pattern), flags
+        args_needed = _named_args[args_key]
+
+        # Are we being provided with its required keyword arguments?
+        args_supplied = set()
+        if args_needed:
+            for k, v in args_needed:
+                try:
+                    args_supplied.add((k, frozenset(kwargs[k])))
+                except KeyError:
+                    raise error("missing named list")
+
+        args_supplied = frozenset(args_supplied)
+
+        # Have we already seen this regular expression and named list?
+        pattern_key = (pattern, type(pattern), flags, args_supplied,
+          DEFAULT_VERSION)
+        return _cache[pattern_key]
+    except KeyError:
+        # It's a new pattern, or new named list for a known pattern.
         pass
-    elif isinstance(pattern, pattern_type):
+
+    # Guess the encoding from the class of the pattern string.
+    if isinstance(pattern, str):
+        guess_encoding = UNICODE
+    elif isinstance(pattern, bytes):
+        guess_encoding = ASCII
+    elif isinstance(pattern, _pattern_type):
         if flags:
             raise ValueError("can't process flags argument with a compiled pattern")
+
         return pattern
     else:
         raise TypeError("first argument must be a string or compiled pattern")
 
-    if kwargs is None:
-        kwargs = {}
-
     # Set the default version in the core code in case it has been changed.
     _regex_core.DEFAULT_VERSION = DEFAULT_VERSION
 
-    # Do we know what keyword arguments are needed?
-    args_key = pattern, type(pattern), flags
-    args_needed = _named_args.get(args_key)
-    if args_needed is not None:
-        # Are we being provided with those keyword arguments?
-        args_supplied = set()
-        for k, v in args_needed:
-            if k not in kwargs:
-                raise error("missing named list")
-            args_supplied.add((k, frozenset(kwargs[k])))
-
-        # Have we already seen this regular expression and named list?
-        pattern_key = (pattern, type(pattern), flags, frozenset(args_supplied),
-          DEFAULT_VERSION)
-        compiled_pattern = _cache.get(pattern_key)
-        if compiled_pattern:
-            return compiled_pattern
-
-    # It's a new pattern (or new named list for a known pattern).
-
-    # Guess the encoding from the class of the pattern string.
-    if isinstance(pattern, bytes):
-        guess_encoding = ASCII
-    else:
-        guess_encoding = UNICODE
-
-    # Parse the pattern. In the old behaviour the inline flags are global and
-    # the pattern will need to be reparsed if a flag becomes turned on.
-    version = (flags & ALL_VERSIONS) or DEFAULT_VERSION
-    global_flags = flags | _regex_core.DEFAULT_FLAGS.get(version, 0)
+    caught_exception = None
 
     while True:
         try:
-            source = Source(pattern)
-            info = Info(global_flags, source.char_type, kwargs)
+            source = _Source(pattern)
+            info = _Info(flags, source.char_type, kwargs)
             info.guess_encoding = guess_encoding
             source.ignore_space = bool(info.flags & VERBOSE)
-            parsed = parse_pattern(source, info)
+            parsed = _parse_pattern(source, info)
             break
-        except UnscopedFlagSet as e:
+        except _UnscopedFlagSet:
             # Remember the global flags for the next attempt.
-            global_flags = e.global_flags | flags
+            flags = info.global_flags
+        except error as e:
+            caught_exception = e
+
+        if caught_exception:
+            raise error(str(caught_exception))
 
     if not source.at_end():
-        raise error("trailing characters in pattern")
+        raise error("trailing characters in pattern at position {}".format(source.pos))
 
     # Check the global flags for conflicts.
-    version = (info.flags & ALL_VERSIONS) or DEFAULT_VERSION
+    version = (info.flags & _ALL_VERSIONS) or DEFAULT_VERSION
     if version not in (0, VERSION0, VERSION1):
         raise ValueError("VERSION0 and VERSION1 flags are mutually incompatible")
 
-    if (info.flags & ALL_ENCODINGS) not in (0, ASCII, LOCALE, UNICODE):
+    if (info.flags & _ALL_ENCODINGS) not in (0, ASCII, LOCALE, UNICODE):
         raise ValueError("ASCII, LOCALE and UNICODE flags are mutually incompatible")
 
     if isinstance(pattern, bytes) and (info.flags & UNICODE):
         raise ValueError("can't use UNICODE flag with a bytes pattern")
-    if not (info.flags & ALL_ENCODINGS):
+
+    if not (info.flags & _ALL_ENCODINGS):
         if isinstance(pattern, str):
             info.flags |= UNICODE
         else:
             info.flags |= ASCII
 
     reverse = bool(info.flags & REVERSE)
-    fuzzy = isinstance(parsed, Fuzzy)
+    fuzzy = isinstance(parsed, _Fuzzy)
+
+    # Should we print the parsed pattern?
+    if flags & DEBUG:
+        parsed.dump(indent=0, reverse=reverse)
 
     # Fix the group references.
     parsed.fix_groups(reverse, False)
@@ -467,7 +514,7 @@ def _compile(pattern, flags=0, kwargs=None):
     parsed = parsed.pack_characters(info)
 
     # Get the required string.
-    req_offset, req_chars, req_flags = get_required_string(parsed, info.flags)
+    req_offset, req_chars, req_flags = _get_required_string(parsed, info.flags)
 
     # Build the named lists.
     named_lists = {}
@@ -477,19 +524,15 @@ def _compile(pattern, flags=0, kwargs=None):
         name, case_flags = key
         values = frozenset(kwargs[name])
         if case_flags:
-            items = frozenset(fold_case(info, v) for v in values)
+            items = frozenset(_fold_case(info, v) for v in values)
         else:
             items = values
         named_lists[name] = values
         named_list_indexes[index] = items
         args_needed.add((name, values))
 
-    # Should we print the parsed pattern?
-    if flags & DEBUG:
-        parsed.dump(indent=0, reverse=reverse)
-
     # Check the features of the groups.
-    check_group_features(info, parsed)
+    _check_group_features(info, parsed)
 
     # Compile the parsed pattern. The result is a list of tuples.
     code = parsed.compile(reverse)
@@ -498,25 +541,25 @@ def _compile(pattern, flags=0, kwargs=None):
     key = (0, reverse, fuzzy)
     ref = info.call_refs.get(key)
     if ref is not None:
-        code = [(OP.CALL_REF, ref)] + code + [(OP.END, )]
+        code = [(_OP.CALL_REF, ref)] + code + [(_OP.END, )]
 
     # Add the final 'success' opcode.
-    code += [(OP.SUCCESS, )]
+    code += [(_OP.SUCCESS, )]
 
     # Compile the additional copies of the groups that we need.
     for group, rev, fuz in info.additional_groups:
         code += group.compile(rev, fuz)
 
     # Flatten the code into a list of ints.
-    code = flatten_code(code)
+    code = _flatten_code(code)
 
     if not parsed.has_simple_start():
         # Get the first set, if possible.
         try:
-            fs_code = compile_firstset(info, parsed.get_firstset(reverse))
-            fs_code = flatten_code(fs_code)
+            fs_code = _compile_firstset(info, parsed.get_firstset(reverse))
+            fs_code = _flatten_code(fs_code)
             code = fs_code + code
-        except FirstSetError:
+        except _FirstSetError:
             pass
 
     # The named capture groups.
@@ -529,11 +572,12 @@ def _compile(pattern, flags=0, kwargs=None):
     # affect the code generation but _are_ needed by the PatternObject.
     compiled_pattern = _regex.compile(pattern, info.flags | version, code,
       info.group_index, index_group, named_lists, named_list_indexes,
-      req_offset, req_chars, req_flags)
+      req_offset, req_chars, req_flags, info.group_count)
 
     # Do we need to reduce the size of the cache?
     if len(_cache) >= _MAXCACHE:
-        shrink_cache(_cache, _named_args, _MAXCACHE)
+        with _cache_lock:
+            _shrink_cache(_cache, _named_args, _MAXCACHE)
 
     args_needed = frozenset(args_needed)
 
@@ -546,11 +590,21 @@ def _compile(pattern, flags=0, kwargs=None):
 
     return compiled_pattern
 
-def compile_replacement(pattern, template):
+def _compile_replacement_helper(pattern, template):
     "Compiles a replacement template."
     # This function is called by the _regex module.
+
+    # Have we seen this before?
+    key = pattern.pattern, pattern.flags, template
+    compiled = _replacement_cache.get(key)
+    if compiled is not None:
+        return compiled
+
+    if len(_replacement_cache) >= _MAXREPCACHE:
+        _replacement_cache.clear()
+
     is_unicode = isinstance(template, str)
-    source = Source(template)
+    source = _Source(template)
     if is_unicode:
         def make_string(char_codes):
             return "".join(chr(c) for c in char_codes)
@@ -565,10 +619,10 @@ def compile_replacement(pattern, template):
         if not ch:
             break
         if ch == "\\":
-            # 'compile_repl_escape' will return either an int group reference
+            # '_compile_replacement' will return either an int group reference
             # or a string literal. It returns items (plural) in order to handle
             # a 2-character literal (an invalid escape sequence).
-            is_group, items = compile_repl_escape(source, pattern, is_unicode)
+            is_group, items = _compile_replacement(source, pattern, is_unicode)
             if is_group:
                 # It's a group, so first flush the literal.
                 if literal:
@@ -583,19 +637,22 @@ def compile_replacement(pattern, template):
     # Flush the literal.
     if literal:
         compiled.append(make_string(literal))
+
+    _replacement_cache[key] = compiled
+
     return compiled
 
-# We define pattern_type here after all the support objects have been defined.
-pattern_type = type(_compile("", 0, {}))
+# We define _pattern_type here after all the support objects have been defined.
+_pattern_type = type(_compile("", 0, {}))
 
 # We'll define an alias for the 'compile' function so that the repr of a
 # pattern object is eval-able.
 Regex = compile
 
 # Register myself for pickling.
-import copyreg
+import copyreg as _copy_reg
 
-def pickle(p):
+def _pickle(p):
     return _compile, (p.pattern, p.flags)
 
-copyreg.pickle(pattern_type, pickle, _compile)
+_copy_reg.pickle(_pattern_type, _pickle, _compile)
